#!/bin/sh
#\
exec wish "$0"  "$@"




# The 4 parameters, program_file, printer_port, temp_dir and default_dir
# described below must be correctly set up for your system.
# Every line beginning with # is a comment, and has no effect.
# The non-commented lines below establish reasonable defaults that
# will probably work most of the time. If you have a weird system,
# you may need to change them

if {$tcl_platform(platform) == "windows"} {

# set plausible windows values

# ...............................................  Start of windows

# The path & name of the non-interactive spin program
# Assume it's somewhere on the default path, else specify, e.g.
# set program_file "c:/nmr/bspinwin"
# note that unix-style forward slashes should be used, not back slashes

set program_file "bspinwin"

# ...............................................

# We print text output by copying to a printer port. May not work
# on all systems. Change the following line when not appropriate
# Windoze will typically need a form feed after ascii print, linux not

set printer_port lpt1
set form_feed 1

# ...............................................

# directory for temporary files (no trailing slash). A ramdisk
# improves speed (about 250k needed for largest problems)
# g: is ramdisk on my system. Modify the following as required

set temp_dir "c:/temp"
#set temp_dir "g:"

# ...............................................

# default directory for input and output files
set default_dir "c:/nmr"

# ............................................... End of Windows

} else {

# set plausible Linux values

# ............................................... Start of Linux

set program_file "bspinlnx"

# ...............................................

# if you are using apsfilter, lpr -Praw is faster
# otherwise it may be a bad idea
# set printer_port "|lpr"
set printer_port "|lpr -Praw"
set form_feed 0

# ...............................................

set temp_dir "/tmp"

# ...............................................

set default_dir "~"

# ............................................... End of Linux
}



# You might optionally change some of these

# display colours and fonts

option add *background "#c1e1ff"
option add *Button.background "#c0c4c0"
option add *Button.foreground sienna4
option add *Button.borderWidth 1
option add *Button.padY 0
option add *Button.font "helvetica 10 bold"
option add *Entry.background white
option add *Entry.font "courier 10 bold"
option add *Text.font "courier 10 bold"
option add *Label.font "helvetica 10 bold"

set plot_background grey75
set plot_axis_colour black
set plot_data_colour blue
set expand_colour red
set axis_font {Helvetica 9 bold}
set info_font {Times 8 italic}



# in a stick plot, lines at the same frequency within this many
# screen pels will be combined
set stick_plot_fuzz 1


####################################################################
# Don't screw around below this,unless you know what you're doing  #
####################################################################


bind Button <KeyPress-Return> {%W invoke}


proc main {} {

#  the main program

    global button_wait ppm nspins s_height s_width global_scale plot_full
    global infilename outfilename linewidth plot_no prev_left prev_right id
    global use_Tkprint tcl_platform tcl_version

    wm iconify .
    set s_width [winfo screenwidth .]
    set s_height [winfo screenheight .]

    set use_Tkprint 0
    if {$tcl_platform(platform) == "windows" && $tcl_version >= 8.2} {

        if {0 == [catch {package require Tkprint 1.1}] } {
            set use_Tkprint 1
        }
    }

    set button_wait 0
    set linewidth 0
    set plot_no 0
    set prev_left 0
    set prev_right 0

    set state 1
    while {$state > 0} {
        set ppm "P"

        set data_source [get_data_source]

        if {($data_source == 2) || ($data_source < 0)} {exit}

        if {$data_source == 1} {
            if {[read_user_file] == -1} {
                tk_messageBox -type ok -message "Error reading file"
                continue;
            }
            for {set i 1} {$i <= $nspins} {incr i} {set id($i) 0}
        }
        destroy .dsin
        set state 2
        if {$data_source == 0} {
            set nspins 2
            inputn
            default_parms
        }

        while {$state > 1} {
            set infilename [tempfilename]
            if {$data_source == 0} { inputparms }
            set data_source 0
# so if user chooses "new parms" it will be interactive,
# even if 1st parms came from a file

            build_input_file $infilename
            set outfilename [tempfilename]
            calculate $infilename $outfilename

            set plot_full 1
            set global_scale 1
# first plot after a new calculation always shows full data

            set state 3

            while {$state > 2} {
                if {[winfo exists .goutwin.c]} {
                    .goutwin.c delete all
                }
                set state [output $outfilename]

# output returns 1 for new calc, 2 to change parms
# 3 to re-plot or 4 to quit

                set plot_full 0
# so user can manipulate the plot width & press the "replot" button

                if {$state >=4} {
                    if {[winfo exists .goutwin]} {destroy .goutwin}
                    break
                }
# loop for re-plot
            }
            file delete $infilename
            file delete $outfilename
            if {$state >= 4} {break}
# loop for new parms
        }
        if {$state >= 4} {break}
# loop for new calculation
    }
    return
}
# end of main program


proc isnumeric {s} {
    if {1 == [scan $s "%g%c" d1 d2]} {
        return 1
    } else {
        return 0
    }
}



proc rnh {x} {
# return a round number > x

    set ax $x
    if {$x < 0} {set ax [expr -$x]}
    if {$ax < 1e-10} {return 0}

    set p [expr log10($ax)]
    set j [expr int([expr ($p < 0)? [expr $p - 1] : $p])]
    set t [expr pow(10, $j)]
    set r [expr $x/$t]
    set j $r
    if {$r >= 0} {set j [expr $r + 1]}
    set j [expr int($j)]

    if {$j == 2 && $r <= 1.5} {return [expr 1.5 * $t]}
    if {$j == -1 && $r <= -1.5} {return [expr -1.5 * $t]}
    if {$j == 7} {set j 8}
    if {$j == 9} {set j 10}
    if {$j == -7} {set j -6}
    if {$j == -9} {set j -8}
    return [expr $j * $t]
}


proc rnl {x} {
# return a round number < x
    return [expr -[rnh [expr -$x]]]
}



proc tempfilename {} {

    global temp_dir

    set base "$temp_dir/sp@@@"
    for {set i 0} {$i <= 999} {incr i} {
        set name "$base$i"
        if {![file exists $name]} {return $name}
    }
    tk_messageBox -type ok -message "DISASTER!\nCan't build temporary file name"
    exit
}


proc get_data_source {} {
    global dsr
    toplevel .dsin

    label .dsin.l -text "What is the source of input data?" -font {Helvetica 12 bold}
    button .dsin.bi -text "Interactive" -width 11 -command {set dsr 0; incr button_wait}
    button .dsin.br -text "Read File" -width 11 -command {set dsr 1; incr button_wait}
    button .dsin.bq -text "Quit" -width 11 -command {set dsr 2; incr button_wait}
    pack .dsin.l -side top -expand 1 -pady 16 -padx 8
    pack .dsin.bi -side left -expand 1  -pady 2 -padx 2
    pack .dsin.br -side left -expand 1  -pady 2 -padx 2
    pack .dsin.bq -side left -expand 1  -pady 2 -padx 2

    centre .dsin

    wm title .dsin "SPIN Input"
    focus -force .dsin.bi
    update

    tkwait variable button_wait
#    destroy .dsin
    return $dsr
}



proc centre {win} {

    global wtemp tcl_platform

    set wtemp $win


    if { $tcl_platform(os) == "Win32s" } {
        after 50 {
            set xm [winfo screenwidth .]
            set ym [winfo screenheight .]
            set x [expr ($xm - [winfo reqwidth "$wtemp"]) / 2]
            set y [expr ($ym - [winfo reqheight "$wtemp"] ) / 3]
            update idletasks
            wm geometry $wtemp "+$x+$y"
        }

    } else {
        after idle {
            update idletasks
            set xm [winfo screenwidth .]
            set ym [winfo screenheight .]
            set x [expr ($xm - [winfo reqwidth "$wtemp"]) / 2]
            set y [expr ($ym - [winfo reqheight "$wtemp"] ) / 3]
            wm geometry $wtemp "+$x+$y"
        }
    }

# the textbook thing doesn't work on win3. Compromise!
}


proc inputn {} {


    global nspins b_background
    global frequency coupling ppm id
    global button_wait

    toplevel .inwin

    frame .inwin.a
    label .inwin.a.nsl -text "Number of spins"
    entry .inwin.a.nse -textvariable nspins -width 2
    pack .inwin.a.nsl -side left -padx 8
    pack .inwin.a.nse -side left -padx 8
    pack .inwin.a -pady 16 -padx 16
    focus -force .inwin.a.nse
    eval .inwin.a.nse selection range 0 end

    frame .inwin.b
    button .inwin.b.ib1 -text "Continue" -underline 0 -width 8 -command {
            if {($nspins >= 2) && ($nspins <= 9)} {
                for {set i 1} {$i <= $nspins} {incr i} {set id($i) 0}
                incr button_wait
            } else {
                tk_messageBox -message "Must have 2 to 9 spins"\
                    -type ok
              }
    }


    button .inwin.b.ib2 -text "Quit" -width 8 -command exit
    pack .inwin.b.ib1 -side left -expand 1 -padx 2
    pack .inwin.b.ib2 -side left -expand 1 -padx 2
    pack .inwin.b -pady 2 -expand 1 -fill x

    bind .inwin <Alt-KeyPress-c> {.inwin.b.ib1 invoke}
    bind .inwin.a.nse <Return> {.inwin.b.ib1 invoke}

    centre .inwin

    wm title .inwin "SPIN Input"

    tkwait variable button_wait
    destroy .inwin
}


proc test_parms {} {

# see if parameter values are ok

global nspins frequency coupling spmhz button_wait

    for {set i 1} {$i <= $nspins} {incr i} {
        if {![isnumeric $frequency($i)]} {
            tk_messageBox -message "f$i is not numeric" -type ok
            focus -force ".inwin.fe($i)"
            eval .inwin.fe($i) selection range 0 end
            return
        }
        for {set j [expr $i + 1]} {$j <= $nspins} {incr j} {
            if {![isnumeric $coupling($i$j)]} {
                tk_messageBox -message "J$i$j is not numeric" -type ok
                eval .inwin.je($i$j) selection range 0 end
                 focus -force ".inwin.je($i$j)"
                return
            }
        }
    }
    if {(![isnumeric $spmhz]) || ($spmhz <= 0)} {
        tk_messageBox -type ok -message\
                "Spectrometer frequency must be\nnumeric and > 0"
        return
    }
    incr button_wait
}


proc default_parms {} {

# set initial default parameter values

    global frequency coupling spmhz nspins id

    for {set i 1} {$i <= $nspins} {incr i} {
        set frequency($i) 0
        set id($i) 0
        for {set j [expr $i + 1]} {$j <= $nspins} {incr j} {
            set coupling($i$j) 0
        }
    }
    set spmhz 100
}



proc doids {} {

# fill in frequencies & couplings of identical spins

    global nspins
    global frequency coupling ppm spmhz id

# build equivalence lists

    for {set i 1} {$i <= $nspins} {incr i} { set elist($i) {} }
    set id(0) 0
    for {set i 1}  {$i < $nspins} {incr i} {
        set j [expr $i + 1]
        if {$id($j)} {
            lappend elist($i) $j
            lappend elist($j) $i
            set k $i
            while {$id($k)} {
                set km1 [expr $k - 1]
                lappend elist($km1) $j
                lappend elist($j) $km1
                incr k -1
            }
        }
    }
#for {set i 1} {$i <= $nspins} {incr i} {puts $elist($i)}
# process equivalence lists

    for {set i 1} {$i <= $nspins} {incr i} {
        .inwin.fe($i) configure -bg white -state normal -takefocus 1
        set ip1 [expr $i + 1]
        for {set j $ip1} {$j <= $nspins} {incr j} {
            .inwin.je($i$j) configure -bg white -state normal -takefocus 1
        }
    }

    for {set i 1} {$i < $nspins} {incr i} {
        set ip1 [expr $i + 1]
        for {set j $ip1} {$j <= $nspins} {incr j} {
            if {-1 != [lsearch $elist($i) $j]} {
                set frequency($j) $frequency($i)
                .inwin.fe($j) configure -bg grey80 -state disabled -takefocus 0
                .inwin.je($i$j) configure -bg #efcfcf -state normal -takefocus 0
                for {set k [expr $j + 1]} {$k <= $nspins} {incr k} {
                    set coupling($j$k) $coupling($i$k)
                    .inwin.je($j$k) configure -bg grey80 -state disabled -takefocus 0
                    continue
                }
            }
            for {set k [expr $j + 1]} {$k <= $nspins} {incr k} {
                if {-1 != [lsearch $elist($j) $k]} {
                    set coupling($i$k) $coupling($i$j)
                    .inwin.je($i$k) configure -bg grey80 -state disabled -takefocus 0
                }
            }

        }
    }
}



proc inputparms {} {

# get the frequencies, coupling constants, spectrometer frequency

    global nspins
    global frequency coupling ppm spmhz id
    global button_wait

    toplevel .inwin


# entry boxes for frequencies

    set nsm1 [expr ($nspins == 2)?2:[expr $nspins - 1]]
    for {set i 1} {$i <= $nspins} {incr i} {
#        set frequency($i) 0
        entry .inwin.fe($i) -textvariable frequency($i) -width 8
        label .inwin.fl($i) -text " f$i"
        grid .inwin.fl($i) -row $i -column 3
        grid .inwin.fe($i) -row $i -column 4 -columnspan 2 -padx 2
        checkbutton .inwin.cb($i) -variable id($i) -command doids -takefocus 0
        grid .inwin.cb($i) -row $i -column 6
    }
    label .inwin.lid -text "Id"
    grid .inwin.lid -row 1 -column 6

# spacer

    frame .inwin.sp -height 8p
    grid .inwin.sp -row [expr $nspins+1]

# entry boxes for couplings

    for {set i 1} {$i < $nspins} {incr i} {
        for {set j [expr $i + 1]} {$j <= $nspins} {incr j} {
#            set coupling($i$j) 0
            entry .inwin.je($i$j) -textvariable coupling($i$j) -width 4
            label .inwin.jl($i$j) -text "J$i$j"
            grid .inwin.jl($i$j) -row [expr {$nspins + $i + 2}] -column [expr {2*$j-3}]
            grid .inwin.je($i$j) -row [expr {$nspins + $i + 2}] -column [expr {2*$j-2}] -padx 2
            bind .inwin.je($i$j) <KeyRelease> doids
        }
        bind .inwin.fe($i) <KeyRelease> doids
    }

# entry box for spec freq

    label .inwin.sfl -text "Spec freqency"
#    set spmhz 100
    entry .inwin.sfe -textvariable spmhz -width 8
    grid .inwin.sfl -row [expr 2*$nspins+2] -column 1 -columnspan 3
    grid .inwin.sfe -row [expr 2*$nspins+2] -column 4 -columnspan 2 -pady 4


#spacer
    frame .inwin.f2 -height 4p
    grid .inwin.f2 -row [expr 2*$nspins+4]

# continue and quit buttons

    frame .inwin.f3
    button .inwin.f3.ib1 -text "Continue" -underline 0 -command {doids;  test_parms; }
    button .inwin.f3.ib2 -text "    Quit    " -command exit

    pack .inwin.f3.ib1 -side left -expand 1
    pack .inwin.f3.ib2 -side left -expand 1
    grid .inwin.f3 -row [expr 2*$nspins+5] -sticky ew -columnspan [expr 2*$nspins + 2]
    bind .inwin <Alt-KeyPress-c> {.inwin.f3.ib1 invoke}

# radio buttons for ppm/Hz

#    set ppm P
    radiobutton .inwin.rp -background cyan -text "ppm" -variable ppm -value P -width 5 -anchor w
    radiobutton .inwin.rh -background cyan -text "Hz" -variable ppm -value H -width 5 -anchor w
    grid .inwin.rp -row 1 -column 1 -columnspan 2
    grid .inwin.rh -row 2 -column 1 -columnspan 2

    centre .inwin

# process any existing identity specifications
    doids

    wm title .inwin "SPIN Input"
    focus -force .inwin.fe(1)
    eval .inwin.fe(1) selection range 0 end

    tkwait variable button_wait
    destroy .inwin
}


proc calculate {input_file_name output_file_name} {

    global temp_dir program_file

    exec $program_file "$input_file_name" "$output_file_name" ">$temp_dir/spin.log"
}


proc results_to_file {f} {
# f is a file which has already been opened, and will be
# closed by caller

    global infilename

    puts $f " Results of SPIN calculation:\n\n INPUT:\n"
    set f1 [open $infilename "r"]
    puts $f [read $f1]
    close $f1

    puts $f " OUTPUT:\n"
    puts $f [.txoutwin.t get 1.0 end]
}


proc file_results {} {

    global default_dir

    set name [tk_getSaveFile -parent .txoutwin -initialdir $default_dir]
    if {$name == ""} {return}

    set f [open $name "w"]
    results_to_file $f
    close $f
}


proc print_results {} {

    global printer_port form_feed

    catch [ set f [open $printer_port "w"]]
    results_to_file $f
    if { $form_feed} { puts $f "\f" }
    close $f
}


proc text_out {result_file} {

    global ppm info_font npeaks sdi sdx spmhz

    toplevel .txoutwin
    wm title .txoutwin "SPIN Output"

    text .txoutwin.t -height 20 -width 36 -background white\
            -yscrollcommand {.txoutwin.sb set}
    frame .txoutwin.f
    label .txoutwin.f.info -text "This display can be edited to remove weak lines" -font $info_font
    button .txoutwin.f.s -text "Save" -width 8 -command {file_results}
    button .txoutwin.f.p -text "Print" -width 8 -command {print_results}
    button .txoutwin.f.d -text "Dismiss" -width 8 -command {destroy .txoutwin}

    pack .txoutwin.f.info -side top -expand 1 -pady 2
    pack .txoutwin.f.s -side left -expand 1 -pady 2
    pack .txoutwin.f.p -side left -expand 1 -pady 2
    pack .txoutwin.f.d -side left -expand 1 -pady 2
    pack .txoutwin.f -side bottom -fill both -expand 1
    pack .txoutwin.t -side left

    scrollbar .txoutwin.sb -command {.txoutwin.t yview}
    pack .txoutwin.sb -side right -fill y

    centre .txoutwin

    if {$ppm == "P"} {
        .txoutwin.t insert end "          Hz        ppm    Intensity\n"
    } else {
        .txoutwin.t insert end "          Hz      Intensity\n"
    }

    for {set i 0} {$i < $npeaks} {incr i} {
        set ip1 [expr {$i + 1}]
        if {$ppm == "P"} {
            set hz [expr {$spmhz * $sdx($i)}]
            .txoutwin.t insert end [format "%3d %10.2f %10.3f  %8.4f\n" $ip1 $hz $sdx($i) $sdi($i)]
        } else {
            .txoutwin.t insert end [format "%3d %10.2f  %8.4f\n" $ip1 $sdx($i) $sdi($i)]
        }
    }

#    set in [open $result_file "r"]
#    .txoutwin.t insert end [read $in]

#    close $in
}


proc draw_axis {left right can} {

    global plot_axis_colour axis_font  axis

    set ticks 6
    set dx [expr $axis(p_width) / double($ticks - 1)]

# axis line
    $can create line $axis(xl) $axis(yt) $axis(xr) $axis(yt) -fill $plot_axis_colour -tags axis

# axis ticks
    for {set i 0} {$i < $ticks} {incr i} {
        set x [expr {round($axis(xl) + $i * $dx)}]
        $can create line $x $axis(yt) $x $axis(yb) -fill $plot_axis_colour -tags axis
    }


# axis numbers
    $can create text $axis(xl) [expr $axis(yb) + 1] -anchor nw\
        -text [format "%.2f" $left] -font $axis_font\
        -fill $plot_axis_colour -tags axnum

    $can create text $axis(xr) [expr $axis(yb) + 1] -anchor ne\
        -text [format "%.2f" $right] -font $axis_font\
        -fill $plot_axis_colour -tags axnum

    for {set i 1} {$i < [expr $ticks-1]} {incr i} {
        set x [expr {$axis(xl) + $i * ($axis(xr) - $axis(xl)) / double($ticks - 1)}]
        set f [expr {$left - $i * ($left - $right) / double($ticks - 1)}]
        $can create text $x [expr {$axis(yb) + 1}] -anchor n\
            -text [format "%.2f" $f] -font $axis_font\
            -fill $plot_axis_colour -tags axnum
    }
}


proc mouse_to_freq {x} {
# calculate the frequencies corresponding to mouse position x

    global axis left right ppm spmhz

    set fp [expr {$left - ($x - $axis(xl)) * ($left - $right) / double($axis(xr) - $axis(xl))}]
    if {$ppm == "P"} {
        set fh [expr {$fp * $spmhz}]
    }   else {
            set fh $fp
            set fp [expr {$fh / double($spmhz)}]
        }

    return "$fh $fp"
}


proc display_freq {x} {
# display the frequency corresponding to mouse position x

    set f [mouse_to_freq $x]
    set h [lindex $f 0]
    set p [lindex $f 1]
    set n [expr {abs($p) > 1 ? 5 : 4}]
    set fstring "%.${n}gHz  %.${n}gppm"
    .goutwin.e.display configure -text [format $fstring $h $p ]
}


proc expand_start {x y} {
    global expand

    set expand(x0) $x
    set expand(y0) $y
}

proc expand_continue {x y} {
    global expand expand_colour

    set expand(x) $x;
    display_freq $x;
    .goutwin.c delete blueline;
    .goutwin.c create line $expand(x0) $expand(y0) $x $expand(y0)\
            -fill $expand_colour -tags blueline
}

proc expand_finish {x y} {

    global expand left right full_left full_right ppm prev_left prev_right

    .goutwin.c delete blueline
    set f [mouse_to_freq $x]
    set fh [lindex $f 0]
    set fp [lindex $f 1]
    set f [mouse_to_freq $expand(x0)]
    set fh0 [lindex $f 0]
    set fp0 [lindex $f 1]

    if {abs($x - $expand(x0)) < 2 && abs($y - $expand(y0)) < 2} {
        set left $full_left
        set right $full_right
    }   else {
            if {$ppm == "P"} {
                set left $fp0
                set right $fp
            }   else {
                    set left $fh0
                    set right $fh
                }
            set prev_left $left
            set prev_right $right
        }
    .goutwin.b.br invoke
# re-plot
}



proc plot_lorentz {result_file can} {

    global program_file
    global ppm  full_right  full_left  right left  spmhz
    global npeaks sdi sdx temp_dir
    global global_scale  linewidth  axis  plot_data_colour

#set t1 [clock clicks]
    set baseline [expr $axis(yt) - 1]
    set h [expr $global_scale * $baseline]
    set lfile [tempfilename]
    exec $program_file -L${ppm}${spmhz} "$result_file" "$lfile"\
        $left $right $axis(p_width) $linewidth $h $baseline\
        $axis(xl) ">$temp_dir/spin.log"

#set t2 [clock clicks]

    set f [open $lfile "r"]
    set ldata [read -nonewline $f]
    close $f
    file delete $lfile

    set ldata [split $ldata]

#set t3 [clock clicks]

        eval $can create line $ldata\
                -fill $plot_data_colour -tags lorentz

#set t4 [clock clicks]
#puts "[expr $t2 - $t1] [expr $t3 - $t2] [expr $t4 - $t3]"
}



proc get_left_right {result_file} {

    global ppm full_right full_left right left plot_full
    global npeaks sdx sdi

    set npeaks 0

    set f [open "$result_file" "r"]
    set k 0
    if {$ppm == "P"} {set k 1}
    set kp1 [expr $k + 1]

    while {1} {
        set s [gets $f]
        if {[eof $f]} {break}
        set x [lindex $s $k]
        set i [lindex $s $kp1]
        set sdx($npeaks) $x
        set sdi($npeaks) $i
        incr npeaks
    }
    set xmax $x
    close $f


    set full_right $sdx(0)
# NMR! smallest freq (start of data list) is plotted on right

    set full_right [rnl $full_right]
    set full_left $xmax
    set full_left [rnh $full_left]
    if {$full_right > 0 && [expr $full_left / $full_right] > 5} {
        set full_right 0
    }

    if {$plot_full} {
        set left $full_left
        set right $full_right
    }
    if {$left < $right} {
        set temp $left
        set left $right
        set right $temp
    }
    if {$left == $right} {set left [expr $right + 10]}


}


proc plot_sticks {result_file can} {

    global ppm full_right full_left right left stick_plot_fuzz
    global npeaks sdi sdx
    global global_scale axis plot_data_colour

# NB SPIN sorts transitions in increasing frequency,
# and consolidates lines of effectively the same frequency

# combine lines that are too close to plot
    set dxmin [expr $stick_plot_fuzz * ($left - $right) / double($axis(p_width))]
# the smallest screen difference possible
    set imax 0
    set xprev 1e44
    set n 0

    for {set j 0} {$j < $npeaks} {incr j} {
        set x $sdx($j)
        set i $sdi($j)

        if {[expr {abs($x - $xprev)}] > $dxmin} {
            set sdx($n) $x
            set sdi($n) $i
            incr n
            set xprev $x
        }   else {
                set i [expr {$sdi([expr {$n - 1}]) + $i}]
                set sdi([expr {$n - 1}]) $i
# accumulate intensities of close peaks
            }
        if {$i > $imax && $x < $left && $x > $right} {set imax $i}
    }

    set npeaks $n


# the sticks
    if { $imax > 0} {
        set y0 [expr $axis(yt) - 1]
        set yscale [expr $global_scale * $y0 / double($imax)]
        set xscale [expr ($axis(xr) - $axis(xl)) / double($left - $right)]
        set aqq [expr $axis(xl) + $xscale * $left]

        for {set j 0} {$j < $npeaks} {incr j}  {
            set x $sdx($j)
            set i $sdi($j)
            if {$x > $left} {break}
            if {$x < $right} {continue}
            set x [expr {$aqq - $xscale * $x}]
            set i [expr {$y0 - $yscale * $i}]
            if {abs($i - $y0) < 0.5} {continue}
            $can create line $x $y0 $x $i -tags stick
        }

        $can itemconfigure stick -fill $plot_data_colour
    }

}


proc previous_expansion {} {
# re-draw using the most recent axis expansion

   global left right prev_left prev_right

   if {$prev_left == $prev_right} {return}

   set left $prev_left
   set right $prev_right
  .goutwin.b.br invoke
}


proc conditional_replot {} {
# re-draw iff window has changed size

    global old_width old_height

    set w [winfo width .goutwin]
    set h [winfo height .goutwin]

    if { $w != $old_width || $h != $old_height } {
        set old_width $w
        set old_height $h
        .goutwin.b.br invoke
    }
}


proc graph_out {result_file} {

    global s_height s_width ppm plot_background plot_full
    global return_state prev_left prev_right
    global left  right  full_left  full_right  axis  linewidth
    global bitmaps  old_width  old_height

    if {![winfo exists .goutwin]} {
        toplevel .goutwin -bd 0
        set w_height [expr $s_height * .75]
        set w_width [expr $s_width - 8]
        set old_width $w_width
        set old_height $w_height
        wm geometry .goutwin [format "%.0fx%.0f+0+0" $w_width $w_height]
    } else {
        set w_width [winfo width .goutwin]
        set w_height [winfo height .goutwin]
    }
# at pgm startup we make the graph window full screen width.
# and 3/4 height. Once it exists, we work with its present size

    wm title .goutwin "SPIN Output"


# fill screen, allowing for width of frame, which -borderwidth
# doesn't seem to suppress on windoze


    set c_width [expr $w_width]
    set axis(c_height) [expr 0.9 * $w_height - .07 * $s_height]

    if {[winfo exists .goutwin.c]} {
        .goutwin.c configure -width $c_width -height $axis(c_height)
        set sw [expr round(88 - 66 * $s_width / double($c_width))]
        if {$sw < 0} {set sw 0}
        .goutwin.e.spacer configure -width $sw
    }

if {![winfo exists .goutwin.c]} {
    canvas .goutwin.c -cursor "crosshair" -bd 0 -width $c_width -height $axis(c_height) -bg $plot_background -highlightthickness 0
    pack .goutwin.c -side top
    bind .goutwin.c <Motion> {display_freq %x}
    bind .goutwin.c <Leave> {.goutwin.e.display configure -text ""}
    bind .goutwin.c <ButtonPress-3> {expand_start %x %y}
    bind .goutwin.c <ButtonRelease-3> {expand_finish %x %y}
    bind .goutwin.c <Button3-Motion> {expand_continue %x %y}
# On a 2-button mouse, right button is 3.
#    .goutwin.c bind all <KeyPress-Right> {previous_expansion}
#    .goutwin.c bind all <KeyPress-Left> { set left $full_left;  set right $full_right;  .goutwin.b.br invoke }
}
if {![winfo exists .goutwin.e]} {
    frame .goutwin.e
    label .goutwin.e.scale -width 4 -anchor w -textvariable global_scale
    label .goutwin.e.spacer -width 22 -anchor w
    button .goutwin.e.scaleup -image $bitmaps(ua) -command {set global_scale [expr 2 * $global_scale]; .goutwin.b.br invoke} -highlightthickness 3

    button .goutwin.e.scaledn -image $bitmaps(da) -command {set hgs [expr {0.5 * $global_scale}];  set global_scale [expr {$hgs >= 1 ? int($hgs) : $hgs}]; .goutwin.b.br invoke} -highlightthickness 3
    label .goutwin.e.display -anchor w -relief sunken -padx 2 -pady 3  -text " " -width 18
    label .goutwin.e.lr -text "Right    " -anchor w
    label .goutwin.e.ll -text "Left" -anchor e
    label .goutwin.e.lw -text "Width" -anchor e
    label .goutwin.e.lh -text "Hz  " -anchor w
    entry .goutwin.e.ew -width 3 -textvariable linewidth
    entry .goutwin.e.el -width 5 -textvariable left
    entry .goutwin.e.er -width 5 -textvariable right

    pack .goutwin.e.display -side left -padx 4
    pack .goutwin.e.spacer -side left
    pack .goutwin.e.lw -side left -padx 2
    pack .goutwin.e.ew -side left
    pack .goutwin.e.lh -side left
    pack .goutwin.e.ll -side left -padx 2
    pack .goutwin.e.ll -side left -padx 2
    pack .goutwin.e.el -side left -padx 2
    pack .goutwin.e.er -side left -padx 2
    pack .goutwin.e.lr -side left -padx 2
    pack .goutwin.e.scaledn -side left -padx 2
    pack .goutwin.e.scaleup -side left -padx 2
    pack .goutwin.e.scale -side left -padx 4

    pack .goutwin.e -side top -fill both -expand 1
    bind .goutwin.e.er <Return> {.goutwin.b.br invoke}
    bind .goutwin.e.el <Return> {.goutwin.b.br invoke}
    bind .goutwin.e.ew <Return> {.goutwin.b.br invoke}
}
if {![winfo exists .goutwin.b]} {
    frame .goutwin.b
    button .goutwin.b.bn -text "New Calc." -width 9 -command {set return_state 1; incr button_wait}
    button .goutwin.b.bp -text "New Parms" -width 9 -command {set return_state 2; incr button_wait}
    button .goutwin.b.bpl -text "Plot Out" -width 9 -command {plot_menu}
    button .goutwin.b.no -text "Num. Out" -width 9 -command "text_out $result_file"
    button .goutwin.b.br -text "Re-plot" -width 9 -command {
         set return_state 3
         if {$left != $full_left || $right != $full_right} {
            set prev_left $left
            set prev_right $right
         }
         incr button_wait
      }
    button .goutwin.b.bq -text "Quit" -width 10 -command {set return_state 4; incr button_wait}

    pack .goutwin.b.bn -side left -expand 1
    pack .goutwin.b.bp -side left -expand 1
    pack .goutwin.b.bpl -side left -expand 1
    pack .goutwin.b.no -side left -expand 1
    pack .goutwin.b.br -side left -expand 1
    pack .goutwin.b.bq -side left -expand 1
    pack .goutwin.b -side top -fill both -expand 1

    bind .goutwin.b.br <ButtonPress-3> {previous_expansion}
    bind .goutwin.c <KeyPress-Right> {previous_expansion}
# right click on replot button or right arrow restores previous expansion

    bind .goutwin.c <KeyPress-Left> { set left $full_left;  set right $full_right;  .goutwin.b.br invoke }
    bind .goutwin <KeyPress-Up> {.goutwin.e.scaleup invoke}
    bind .goutwin <KeyPress-Down> {.goutwin.e.scaledn invoke}
}
   update idletasks

    set axis(p_width) [expr $c_width - 8]

# axis dimensions
    set axis(xl) 5
    set axis(xr) [expr $axis(xl) + $axis(p_width)]
    set axis(yt) [expr $axis(c_height) - .06 * $s_height]
    set axis(yb) [expr $axis(c_height) - .04 * $s_height]


    get_left_right $result_file

    if {$linewidth <= 0.001} {
        plot_sticks $result_file .goutwin.c
    } else {
        plot_lorentz $result_file .goutwin.c
      }

# must draw axis after plot routine, which sets left and right
# it would really be better to establish left & right earlier

    draw_axis $left $right .goutwin.c

    bind .goutwin <Configure> {conditional_replot}

    tkwait variable button_wait

    return $return_state
}



proc plot_menu {} {

    global pmu_wait tcl_platform

    set pmu_wait 0

    toplevel .pmu
    set fmt [format "+%d+%d" [winfo rootx .goutwin.b.bpl] [winfo rooty .goutwin.b.bpl]]

    if { $tcl_platform(os) == "Win32s" } {update idletasks}
    wm geometry .pmu $fmt
    wm title .pmu ""
    button .pmu.ps -text "Postscript" -command {do_plot; incr pmu_wait}
    button .pmu.cb -text "Clipboard " -command {do_clip; incr pmu_wait}
    button .pmu.can -text "  Cancel  " -command {incr pmu_wait}
    pack .pmu.ps -fill x
    if { $tcl_platform(platform) == "windows" } {pack .pmu.cb -fill x}
    pack .pmu.can -fill x

    tkwait variable pmu_wait
    destroy .pmu
}

proc do_clip {} {
    global use_Tkprint

    if {$use_Tkprint} {

        MetaFile .goutwin.c -clipboard 1
    } else {
        do_clip_the_hard_way
    }
}



proc do_clip_the_hard_way {} {

    global linewidth outfilename left right tcl_platform

    toplevel .gw -bd 0 -relief flat -highlightthickness 0
    wm overrideredirect .gw 1
    update idletasks
    set geom [format "%dx%d+%d+%d" [winfo width .goutwin.c]\
        [winfo height .goutwin.c] [winfo rootx .goutwin.c] [winfo rooty .goutwin.c]]
    wm geometry .gw $geom

    canvas .gw.c -background white -bd 0 -relief flat -highlightthickness 0

    pack .gw.c -fill both -expand 1

    toplevel .plins -bg white
    set fmt [format "+%d+%d" [winfo rootx .goutwin.b.bpl] [winfo rooty .goutwin.b.bpl]]

    if { $tcl_platform(os) == "Win32s" } {update idletasks}
    wm geometry .plins $fmt
    wm title .plins "Message"

    label .plins.msg -text "Press Alt-Print Screen to copy.\nThen press Enter to restore display\n"\
                -bg white -font {helvetica 10 bold}
    pack .plins.msg -fill both -expand 1

    if {$linewidth <= 0.001} {
        plot_sticks $outfilename .gw.c
        .gw.c itemconfigure stick -fill black
    } else {
        plot_lorentz $outfilename .gw.c
        .gw.c itemconfigure lorentz -fill black
      }

    draw_axis $left $right .gw.c
    focus -force .gw
    bind .gw <KeyPress-Return> {destroy .gw;  destroy .plins}
}


proc do_plot {} {

    global plot_no temp_dir axis_font

    set filename "${temp_dir}/plot${plot_no}.ps"
    incr plot_no

    set filename [tk_getSaveFile -initialdir $temp_dir -initialfile $filename]
    set fs [file split $filename]
    set fse [lindex $fs end]
    if { [string match "|*" $fse ] } {
       set filename $fse
    }
# if there's a pipe as the last element, throw away the directory part
# of the filename, assuming user wants something like |lpr

    set fm(Helvetica-Bold) [list Times-Bold 24]

    catch [set f [open [lindex $filename end] "w"]]

    .goutwin.c itemconfigure axnum -font {helvetica 18 bold}
# what looks good on screen is too small on PS

    update idletasks
    .goutwin.c postscript -channel $f -rotate 1 -pagewidth 9i -fontmap fm
    .goutwin.c itemconfigure axnum -font $axis_font
    close $f

}




proc output {result_file} {
    return [graph_out  $result_file]
}


proc build_input_file {infilename} {

    global nspins spmhz ppm frequency coupling

    set infile [open $infilename "w"]
    puts $infile "# Input file for SPIN program"
    puts $infile $nspins
    puts $infile "$ppm   $spmhz"
    for {set i 1} {$i <= $nspins} {incr i} {
        puts -nonewline $infile "$frequency($i)  "
    }
    puts $infile ""
    for {set i 1} {$i < $nspins} {incr i} {
        for {set j [expr $i + 1]} {$j <= $nspins} {incr j} {
            puts -nonewline $infile "$coupling($i$j)  "
        }
        puts $infile ""
    }
    close $infile
}


proc get_line {f} {
# read and return a line from the open file f,
# ignoring blank lines, and those starting with #
# return null string on failure

    while {1} {
        set s [gets $f]
        if {$s == "" && [eof $f]} {return ""}
        set s1 [lindex $s 0]
        if {$s1 == ""} {continue}
        if {[string index $s1 0] == "#"} {continue}
        return $s
    }
}



proc read_user_file {} {
# get a file name from user, open it, read parms.
# return 0 on success, -1 on failure

    global frequency coupling nspins ppm spmhz default_dir

    set name [tk_getOpenFile -initialdir $default_dir -parent .dsin\
                                    -title " SPIN: Select the input file"]
    if {$name == ""} {return -1}

    set f [open $name "r"]

    set s [get_line $f]
    if {$s == ""} {return "-1"}
    set nspins [lindex $s 0]
    if {![isnumeric $nspins]} {return "-1"}
    set s [get_line $f]
    if {$s == ""} {return "-1"}
    set ppm [lindex $s 0]
    set spmhz [lindex $s 1]

    set s [get_line $f]
    if {$s == ""} {return "-1"}
    for {set i 1} {$i <= $nspins} {incr i} {
        set frequency($i) [lindex $s [expr $i - 1]]
        if {![isnumeric $frequency($i)]} {return "-1"}
    }

    for {set i 1} {$i < $nspins} {incr i} {
        set s [get_line $f]
        if {$s == ""} {return "-1"}
        for {set j [expr $i + 1]} {$j <= $nspins} {incr j} {
            set coupling($i$j) [lindex $s [expr {$j - $i - 1}]]
            if {![isnumeric $coupling($i$j)]} {return "-1"}
        }
    }
    close $f
    return "0"
}

# bitmaps for up and down arrows


set uadata {
    #define foo_width 16
    #define foo_height 16
    static char foo_bits[] = {
        0x00, 0x01, 0x00, 0x01, 0x80, 0x03, 0x80, 0x03,
        0xc0, 0x07, 0xc0, 0x07, 0xe0, 0x0f, 0xe0, 0x0f,
        0xf0, 0x1f, 0xf0, 0x1f, 0xf8, 0x3f, 0xf8, 0x3f,
        0xfc, 0x7f, 0xfc, 0x7f, 0xfe, 0xff, 0x00, 0x00};
}

button .zombie
set bfg [option get .zombie foreground Foreground]
destroy .zombie
set bitmaps(ua) [image create bitmap -data $uadata -foreground $bfg]

set dadata {
    #define foo_width 16
    #define foo_height 16
    static char foo_bits[] = {
        0x00, 0x00, 0xfe, 0xff, 0xfc, 0x7f, 0xfc, 0x7f,
        0xf8, 0x3f, 0xf8, 0x3f, 0xf0, 0x1f, 0xf0, 0x1f,
        0xe0, 0x0f, 0xe0, 0x0f, 0xc0, 0x07, 0xc0, 0x07,
        0x80, 0x03, 0x80, 0x03, 0x00, 0x01, 0x00, 0x01};
}
set bitmaps(da) [image create bitmap -data $dadata -foreground $bfg]



main
exit
# execute the main program, quit when it returns

